local gr = love.graphics
local tex = require 'texsphere'

local pi, sin, cos, floor = math.pi, math.sin, math.cos, math.floor
--[[ To use:
1. tex = require 'texsphere'
2. load an texture - gr.newImage( filename )
3.	canvas	= tex.image( image, radius, num-of-repeats ) - best results num>4
4. if light desired, apply light shader to canvas
5.	draw canvas	- gr.draw(canvas,x,y,ang,sx,sy,ox,oy) - ox,oy for centering on x,y
note: background color only available for non-square textures
]]

function love.load()
	love.window.setTitle( 'Textured Circle with Light Effect -"a" to change texture - "i" for stats' )
	star = createStar( 60, 22, 8, {1,1,0}, {.6,.6,0})
	bricks = gr.newImage( 'bricks.png' )
	can2	= tex.image( star, 200, 6, {.5,0,.5}, 'nsqr' )	-- not square (maintain aspect ratio)
	an		= 0
	anim	= {
		run		= false,
		delay		= 0.2,
		lapsed	= 0,
		frame		= 2,
		im			= {}
		}
	for i = 1, 10 do anim.im[i] = gr.newImage( 'stickmen/s'..i..'.png' ) end
	end

function love.update( dt )
	an = an + dt
	if anim.run then
		anim.lapsed = anim.lapsed + dt
		if anim.lapsed > anim.delay then
			anim.lapsed = 0
			anim.frame = anim.frame < #anim.im and anim.frame+1 or 1
			can = tex.image( anim.im[anim.frame], 250, 5, {.4,.4,.6,.9},'fit' )
			local x, y = circlePath( 0.5, 0.5, -an+4*pi/3, 0.5 )
			can = tex.light( can, x, y, 0.4 )
			end
		else
		if not anim.run then	-- keep light stationary as canvas rotates
			local x,y = circlePath( 0.5, 0.5, -an+4*pi/3, .5 )	-- light in upper-left
			can = tex.light( can2, x, y, 0.6 )	-- overwrites previous can
			end
		end
	end

function love.draw()
	gr.setColor( 0.6, 0, 0.6 )
	gr.rectangle( 'fill', 50, 280, 700, 40, 10, 10 )
	gr.setColor( 1, 1, 1 )
	gr.draw( can, 400, 300, an, 1, 1, can:getWidth()/2, can:getHeight()/2 )
	if anim.run then
		gr.draw( anim.im[anim.frame], 20, 450 )
		gr.print( anim.frame, 85-1.75*#tostring(anim.frame), 500 )
		end
	if showStats then tex.stats( 660, 500 ) end
	end

function love.keypressed( key )
	if key == 'escape' then love.event.quit() 	end
	if key == 'a' then
		anim.run = not anim.run
		if not anim.run then
			can2	= tex.image( bricks, 200, 3 )
			end
		end
	if key == 'i' then showStats = not showStats end
	end
-- **************************************************
createStar = function( ro, ri, n, cr, cl, cb )
	local x, y = ro, ro
	local an = 2*pi/n
	local an2 = pi/n
	local po, pr, pl = {}, {}, {}
	for i = 0,  n do
		po[#po+1] = { x + ro * cos(an*i),		y + ro * sin(an*i) }
		pr[#pr+1] = { x + ri * cos(an*i+an2),	y + ri * sin(an*i+an2) }
		pl[#pl+1] = { x + ri * cos(an*i-an2),	y + ri * sin(an*i-an2) }
		end
	starcan = gr.newCanvas( 2*ro, 2*ro )
	gr.setCanvas( starcan )
	if cb then gr.setColor( cb ) gr.rectangle('fill',0,0,2*ro,2*ro) end
		gr.setColor( cr )		-- right side
		for i = 1, n do gr.polygon('fill',x,y,po[i][1],po[i][2],pr[i][1],pr[i][2]) end
		gr.setColor( cl )		-- left side
		for i = 1, n do gr.polygon('fill',x,y,po[i][1],po[i][2],pl[i][1],pl[i][2]) end
		gr.setCanvas()
	return starcan
	end

circlePath = function( cx, cy, ang, r ) -- mode light to compensate for rotation
	return r * cos(ang) + cx, r * sin(ang) + cy
	end
