local gr = love.graphics
local shade = require 'shade'
local texsphere = {}
texsphere.shader = gr.newShader [[
		const number pi = 3.14159265;
		const number pi2 = 2.0 * pi;
		extern number rep;
		vec4 effect(vec4 color, Image texture, vec2 tc, vec2 pixel_coords)
		{
		   vec2 p = 2.0 * (tc - 0.5);					// center in canvas
		   number r = sqrt(p.x*p.x + p.y*p.y);			// sphere radius
		   if (r > 1.0) discard;
		   number d = r != 1.0 ? asin(r) / r : 0.0;
		   vec2 p2 = 2 * d * p * rep;
		   number x3 = mod(p2.x / pi2 + 0.5 , 1.0);
		   number y3 = mod(p2.y / pi2 + 0.5 , 1.0);
		   vec2 newCoord = vec2(x3, y3);					// location of texture on sphere
		   vec4 sphereColor = color * Texel(texture, newCoord);
		   return sphereColor;
		}
		]]
-- for a circle, need a square texture - non-square textures are drawn on square canvas.
texsphere.image = function( im, r, rp, bclr, mode )	-- image,radius,repeats[,clr,shape]
	local iw, ih = im:getWidth(), im:getHeight()
	texsphere.shader:send( 'rep', rp )
	gr.setColor( 1, 1, 1 )
	local mode = mode or 'sqr'		-- sqr expands either w or h to form distorted texture
	local clr = bclr or {0,0,0}
	local img,sx,sy,can = im
	collectgarbage()	-- only need if tex.image continually called
	if mode ~= 'sqr' then		-- maintains aspect ratio of x snd y in square
		local dm = ih > iw and ih or iw	-- determine which dimension is larger
		img = gr.newCanvas( dm, dm )		-- new square canvas (size = larger of w & h)
		gr.setCanvas( img )					-- empty space in square can be colored
			if bclr then gr.setColor(clr) gr.rectangle('fill',0,0,dm,dm) end
			gr.setColor(1,1,1)
			if dm == ih then
				gr.draw( im, math.abs(ih-iw)/2, 0 )	-- center image in x
				else
				gr.draw( im, 0,math.abs(ih-iw)/2 )  -- center image in y
				end
			gr.setCanvas()
		sx, sy = 2*r/dm, 2*r/dm		-- scale texture to fit canvas
		else
		sx, sy = 2*r/iw, 2*r/ih		-- distort texture to fit square
		end
	can = gr.newCanvas( 2*r, 2*r )
	gr.setShader( texsphere.shader )
		gr.setCanvas( can )
			gr.setColor( 1, 1, 1 )
			gr.draw( img, 0, 0, 0, sx, sy )	-- img maybe an image or a canvas
			gr.setCanvas()
		gr.setShader()
	return can
	end

texsphere.light = function( i, x, y, p )	-- image, light location (0-1),amout covered(0 = all, 1 = spot effect
	local ncan = gr.newCanvas( i:getWidth(), i:getHeight() )
	collectgarbage() 		-- minimizes memory accumulation
	shade:send( 'x', x )	-- x,y = .1,.1 for light in upper-left corner
	shade:send( 'y', y )
	shade:send( 'dec', p )	-- darkness
	gr.setShader( shade )
		gr.setCanvas( ncan )
			gr.draw( i )
			gr.setCanvas()
		gr.setShader()
	return ncan
	end

texsphere.stats = function( x, y )
	local st = string.format
	gr.setColor( 0.5, 0.5, 0.5, .6 )
	gr.rectangle( 'fill', x-5, y-5, 140, 100, 10, 10 )
	gr.setColor( 1, 1, 1 )
	gr.rectangle( 'line', x-5, y-5, 140, 100, 10, 10 )
	local stats = love.graphics.getStats( stats )
	gr.print( st( 'canvases: %s', stats.canvases), x, y )
	gr.print( st( 'canvases switches: %s', stats.canvasswitches), x, y+15 )
	gr.print( st( 'draw calls: %s', stats.drawcalls), x, y+30 )
	gr.print( st( 'draw call bactched: %s', stats.drawcallsbatched), x,y+45)
	gr.print( st( 'memory useage: %s', math.floor(collectgarbage("count"))), x, y+60 )
	gr.print( st('fps: %s',love.timer.getFPS( )), x, y+75 )
	end

return texsphere